#ifndef CCONVOLUTION3X3_H
#define CCONVOLUTION3X3_H

#include "IAssignment.h"

#include <string>

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CConvolution3x3
	:public IAssignment
{
protected:
	std::string		m_FileName;
	size_t			m_TileSize[2];

	unsigned int	m_Height;
	unsigned int	m_Width;
	unsigned int	m_Pitch;
	//if true, only one channel is used
	bool			m_Monochrome;

	// host data
	float			m_hConvolutionKernel[3][3];
	float			m_KernelWeight;
	float			m_Offset;

	float*			m_hSourceChannels[3]; //R, G, B channels
	float*			m_hCPUResultChannels[3]; //the convolved image
	float*			m_hGPUResultChannels[3]; //the convolved image

	//we process exactly one channel on the GPU in the same time
	cl_mem			m_dSourceChannels[3];
	cl_mem			m_dResultChannels[3];
	//kernel constants
	cl_mem			m_dKernelConstants;

	cl_program		m_Program;
	cl_kernel		m_ConvolutionKernel;

protected:
	void ConvolutionChannelCPU(unsigned int Channel);
	//the last parameter is for timing
	void ConvolutionChannelGPU(unsigned int Channel, cl_context Context, cl_command_queue CommandQueue, int NIterations);
	void SaveImage(const char* FileName, float* Channels[3]);

public:
	//the file is loaded from the given path, and a single work-group will process
	//a tile of TileSize
	CConvolution3x3(const char* FileName, size_t TileSize[2], float ConvKernel[3][3], bool Monochrome, float Offset);

	virtual ~CConvolution3x3();

	virtual bool InitResources(cl_device_id Device, cl_context Context);
	
	virtual void ReleaseResources();

	virtual void ComputeGPU(cl_context Context, cl_command_queue CommandQueue);

	virtual void ComputeCPU();

	virtual bool ValidateResults();
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif
