#ifndef CCONVOLUTION_BILATERAL_H
#define CCONVOLUTION_BILATERAL_H

#include "IAssignment.h"
#include "Common.h"
#include "Pfm.h"

#include <string>

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CConvolutionBilateral
	:public IAssignment
{
protected:
	std::string		m_FileName;
	std::string		m_NormalFileName;
	std::string		m_DepthFileName;
	//we use different local work sizes during the two convolution kernels
	size_t			m_LocalSizeHorizontal[2];
	size_t			m_LocalSizeVertical[2];
	//how many convolution steps a single thread executes
	int				m_StepsHorizontal;
	int				m_StepsVertical;

	int				m_Height;
	int				m_Width;
	int				m_Pitch;


	//host data
	cl_float4*		m_hNormDepthBuffer;
	float*			m_hKernelHorizontal;
	float*			m_hKernelVertical;
	int	m_KernelRadius;

	float*			m_hSourceChannels[3]; //R, G, B channels

	cl_int*			m_hCPUDiscBuffer;
	cl_int*			m_hGPUDiscBuffer;

	float*			m_hCPUResultChannels[3]; //the convolved image
	float*			m_hGPUResultChannels[3]; //the convolved image

	cl_mem			m_dDiscBuffer;
	cl_mem			m_dNormDepthBuffer;
	cl_mem			m_dResultChannelTemp[3]; // Intermediate result (after horizontal pass)
	cl_mem			m_dResultChannel[3];

	//kernel coefficients
	cl_mem			m_dKernelHorizontal;
	cl_mem			m_dKernelVertical;

	cl_program		m_Program;

	cl_kernel		m_HorizontalDiscKernel;
	cl_kernel		m_VerticalDiscKernel;
	//horizontal convolution pass
	cl_kernel		m_HorizontalKernel;
	//vertical convolution pass
	cl_kernel		m_VerticalKernel;

protected:
	void ConvolutionChannelGPU(unsigned int Channel, cl_context Context, cl_command_queue CommandQueue);
	void ConvolutionChannelCPU(unsigned int Channel);

public:
	//the file is loaded from the given path, and a single work-group will process
	//a tile of TileSize
	CConvolutionBilateral(const char* FileName, const char* NormalFileName, const char* DepthFileName, size_t LocalSizeHorizontal[2], size_t LocalSizeVertical[2],
		int StepsHorizontal, int StepsVertical, int KernelRadius, float* pKernelHorizontal, float* pKernelVertical);

	virtual ~CConvolutionBilateral();

	virtual bool InitResources(cl_device_id Device, cl_context Context);
	
	virtual void ReleaseResources();

	virtual void ComputeCPU();
	virtual void ComputeGPU(cl_context Context, cl_command_queue CommandQueue);

	virtual bool ValidateResults();
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif
