#ifndef CCONVOLUTION_SEPARABLE_H
#define CCONVOLUTION_SEPARABLE_H

#include "IAssignment.h"

#include <string>

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CConvolutionSeparable
	:public IAssignment
{
protected:
	std::string		m_FileName;
	//we use different local work sizes during the two convolution kernels
	size_t			m_LocalSizeHorizontal[2];
	size_t			m_LocalSizeVertical[2];
	//how many convolution steps a single thread executes
	int				m_StepsHorizontal;
	int				m_StepsVertical;

	int				m_Height;
	int				m_Width;
	int				m_Pitch;

	//host data
	float*			m_hKernelHorizontal;
	float*			m_hKernelVertical;
	int	m_KernelRadius;

	float*			m_hSourceChannels[3]; //R, G, B channels
	float*			m_hCPUResultChannels[3]; //the convolved image
	float*			m_hGPUResultChannels[3]; //the convolved image

	//we process exactly one channel on the GPU in the same time...
	//but to avoid separate copies of the data among kernel calls, we still allocate buffers for each channel
	cl_mem			m_dGPUInputBuffers[3];
	cl_mem			m_dGPUOutputBuffers[3];
	cl_mem			m_dGPUWorkingBuffer;

	//kernel coefficients
	cl_mem			m_dKernelHorizontal;
	cl_mem			m_dKernelVertical;

	cl_program		m_Program;
	//horizontal convolution pass
	cl_kernel		m_HorizontalKernel;
	//vertical convolution pass
	cl_kernel		m_VerticalKernel;

protected:
	void ConvolutionChannelCPU(unsigned int Channel);
	void ConvolutionChannelGPU(unsigned int Channel, cl_context Context, cl_command_queue CommandQueue, int NIterations);

public:
	//the file is loaded from the given path, and a single work-group will process
	//a tile of TileSize
	CConvolutionSeparable(const char* FileName, size_t LocalSizeHorizontal[2], size_t LocalSizeVertical[2],
		int StepsHorizontal, int StepsVertical, int KernelRadius, float* pKernelHorizontal, float* pKernelVertical);

	virtual ~CConvolutionSeparable();

	virtual bool InitResources(cl_device_id Device, cl_context Context);
	
	virtual void ReleaseResources();

	virtual void ComputeGPU(cl_context Context, cl_command_queue CommandQueue);

	virtual void ComputeCPU();

	virtual bool ValidateResults();
};

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif